/*

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

import javax.swing.*;
import java.awt.event.*;
import java.util.*;
import java.awt.*;
import java.io.*;

public class Curved extends JFrame implements ActionListener
{
	private static int max_players=6;
	private static int min_points=5;
	private static int max_points=40;
	private static int default_points=40;

	//GUI

	//panels
	private JPanel optionsPnl, playPnl;
	private JTabbedPane tabs;

	//options panel
	private JButton[] colors;
	private JButton[][] keys;
	private JRadioButton[] speed, turn, hole;
	private ButtonGroup gSpeed, gTurn, gHole;
	private JCheckBox fullscreen;
	private JSpinner players,points;
	private JButton save,reload,defaults;
	private JLabel lbl;

	//play panel
	private JButton playOfflineBtn,hostBtn,joinBtn;

	//properties with game config
	private Properties props;

	public Curved()
	{
		super("~ CurveD ~");

		//initialize GUI

		tabs = new JTabbedPane();
		getContentPane().add(tabs);

		//-- options panel--

		optionsPnl = new JPanel();
		optionsPnl.setLayout(new FlowLayout(FlowLayout.LEFT,6,6));
		optionsPnl.setPreferredSize(new Dimension(384,430));

		//colors and key configuration of the players
		keys=new JButton[2][max_players];
		colors=new JButton[max_players];
		optionsPnl.add(Box.createHorizontalStrut(130));
		optionsPnl.add(new JLabel("color             left                            right"));
		for(int i=0;i<max_players;i++)
		{
			lbl=new JLabel("player " + (i+1) + " :");
			lbl.setPreferredSize(new Dimension(130,25));
			optionsPnl.add(lbl);

			colors[i]=new JButton("");
			colors[i].setPreferredSize(new Dimension(25,25));
			colors[i].addActionListener(this);

			keys[0][i]=new JButton("");
			keys[0][i].setPreferredSize(new Dimension(100,25));
			keys[0][i].addActionListener(this);

			keys[1][i]=new JButton("");
			keys[1][i].setPreferredSize(new Dimension(100,25));
			keys[1][i].addActionListener(this);

			optionsPnl.add(colors[i]);
			optionsPnl.add(keys[0][i]);
			optionsPnl.add(keys[1][i]);
		}

		//number of players -> JSpinner : from 2 to max_players
		lbl=new JLabel("# players");
		lbl.setPreferredSize(new Dimension(130,25));
		optionsPnl.add(lbl);
		players=new JSpinner(new SpinnerNumberModel(2,2,max_players,1));
		players.setPreferredSize(new Dimension(40,22));
		optionsPnl.add(players);
		optionsPnl.add(Box.createHorizontalStrut(150));

		//points needed to win the game -> JSpinner : from min_points to max_points
		lbl=new JLabel("# points to win");
		lbl.setPreferredSize(new Dimension(130,25));
		optionsPnl.add(lbl);
		points=new JSpinner(new SpinnerNumberModel(default_points,min_points,max_points,1));
		points.setPreferredSize(new Dimension(40,22));
		optionsPnl.add(points);
		optionsPnl.add(Box.createHorizontalStrut(150));

		//gamespeed: low-normal-high radiobuttons
		lbl=new JLabel("gamespeed");
		lbl.setPreferredSize(new Dimension(130,25));
		optionsPnl.add(lbl);
		speed=new JRadioButton[3];
		speed[0]=new JRadioButton("low");
		speed[1]=new JRadioButton("normal");
		speed[2]=new JRadioButton("high");
		gSpeed=new ButtonGroup();
		gSpeed.add(speed[0]);
		gSpeed.add(speed[1]);
		gSpeed.add(speed[2]);
		optionsPnl.add(speed[0]);
		optionsPnl.add(speed[1]);
		optionsPnl.add(speed[2]);

		//turnspeed: low-normal-high radiobuttons
		lbl=new JLabel("turnspeed");
		lbl.setPreferredSize(new Dimension(130,25));
		optionsPnl.add(lbl);
		turn=new JRadioButton[3];
		turn[0]=new JRadioButton("low");
		turn[1]=new JRadioButton("normal");
		turn[2]=new JRadioButton("high");
		gTurn=new ButtonGroup();
		gTurn.add(turn[0]);
		gTurn.add(turn[1]);
		gTurn.add(turn[2]);
		optionsPnl.add(turn[0]);
		optionsPnl.add(turn[1]);
		optionsPnl.add(turn[2]);

		//amount of holes: low-normal-high radiobuttons
		lbl=new JLabel("amount of holes");
		lbl.setPreferredSize(new Dimension(130,25));
		optionsPnl.add(lbl);
		hole=new JRadioButton[3];
		hole[0]=new JRadioButton("low");
		hole[1]=new JRadioButton("normal");
		hole[2]=new JRadioButton("high");
		gHole=new ButtonGroup();
		gHole.add(hole[0]);
		gHole.add(hole[1]);
		gHole.add(hole[2]);
		optionsPnl.add(hole[0]);
		optionsPnl.add(hole[1]);
		optionsPnl.add(hole[2]);

		//fullscreen: checkbox
		lbl=new JLabel("fullscreen");
		lbl.setPreferredSize(new Dimension(130,25));
		optionsPnl.add(lbl);
		fullscreen = new JCheckBox();
		optionsPnl.add(fullscreen);
		optionsPnl.add(Box.createHorizontalStrut(150));

		//save changes button (saves properties)
		save=new JButton("save changes");
		optionsPnl.add(save);
		save.addActionListener(this);
		optionsPnl.add(Box.createHorizontalStrut(3));

		//cancel changes button (reloads properties)
		reload=new JButton("cancel changes");
		optionsPnl.add(reload);
		reload.addActionListener(this);
		optionsPnl.add(Box.createHorizontalStrut(3));

		//load defaults button (loads default properties)
		defaults=new JButton("load defaults");
		optionsPnl.add(defaults);
		defaults.addActionListener(this);

		//--play CurveD panel--

		playPnl = new JPanel();
		playPnl.setLayout(new FlowLayout(FlowLayout.LEFT,6,6));
		playPnl.setPreferredSize(new Dimension(384,430));

		//start offline game
		playOfflineBtn=new JButton("Play offline game");
		playOfflineBtn.setPreferredSize(new Dimension(150,25));
		playPnl.add(playOfflineBtn);
		playOfflineBtn.addActionListener(this);
		playPnl.add(Box.createHorizontalStrut(200));

		//host online game ( not available yet )
		hostBtn=new JButton("Host online game");
		hostBtn.setPreferredSize(new Dimension(150,25));
		hostBtn.setEnabled(false);
		playPnl.add(hostBtn);
		playPnl.add(Box.createHorizontalStrut(200));

		//join online game ( not available yet )
		joinBtn=new JButton("Join online game");
		joinBtn.setPreferredSize(new Dimension(150,25));
		joinBtn.setEnabled(false);
		playPnl.add(joinBtn);
		playPnl.add(Box.createHorizontalStrut(200));

		//add the panels to the tabbedpane
		tabs.addTab("Game Options", optionsPnl);
		tabs.addTab("Play CurveD", playPnl);

		//window properties
		pack();
		setResizable(false);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setLocationRelativeTo(null);
		show();

		//load the game properties
		loadProperties();
	}

	//event handling
	public void actionPerformed(ActionEvent e)
	{
		//--save changes button--
		if(e.getSource()==save)
		{
			saveProperties();
		}
		//--cancel changes button--
		else if(e.getSource()==reload)
		{
			loadProperties();
		}
		//--load defaults button--
		else if(e.getSource()==defaults)
		{
			loadDefaults();
		}
		//--start offline game button--
		else if(e.getSource()==playOfflineBtn)
		{
			new CurvedOffline();
		}
		else
		{
			for(int i=0; i<max_players; i++)
			{
				//--color buttons--
				if(e.getSource()==colors[i])
				{
					//colorchooser
					Color c = JColorChooser.showDialog(null,"pick a color",colors[i].getBackground());
					if(c!=null)
					{
						colors[i].setBackground(c);
						props.setProperty("color"+(i+1),""+c.getRGB());
					}
				}
				//--left key buttons--
				else if(e.getSource()==keys[0][i])
				{
					keys[0][i].requestFocus();
					//anonymous keyadapter class needs access to i
					//--> so it should be declared final
					final int x=i;
					//add a keylistener to get the desired key
					keys[0][i].addKeyListener(
						new KeyAdapter()
						{
							public void keyReleased(KeyEvent e)
							{
								keys[0][x].setText(KeyEvent.getKeyText(e.getKeyCode()));
								props.setProperty("left"+(x+1),""+e.getKeyCode());
								//remove the keylistener again
								keys[0][x].removeKeyListener(this);
							}
						});
				}
				//--right key buttons--
				else if(e.getSource()==keys[1][i])
				{
					keys[1][i].requestFocus();
					//anonymous keyadapter class needs access to i
					//--> so it should be declared final
					final int x=i;
					//add a keylistener to get the desired key
					keys[1][i].addKeyListener(
						new KeyAdapter()
						{
							public void keyReleased(KeyEvent e)
							{
								keys[1][x].setText(KeyEvent.getKeyText(e.getKeyCode()));
								props.setProperty("right"+(x+1),""+e.getKeyCode());
								//remove the keylistener again
								keys[1][x].removeKeyListener(this);
							}
						});
				}
			}
		}
	}

	//load the game properties
	private void loadProperties()
	{
		try
		{
			props=new Properties();
			//load the properties from the file 'properties'
			props.load(new FileInputStream("properties"));

			//load colors and key configuration
			for(int i=0; i<max_players; i++)
			{
				colors[i].setBackground(new Color(Integer.parseInt(props.getProperty("color"+(i+1)))));
				keys[0][i].setText(KeyEvent.getKeyText(Integer.parseInt(props.getProperty("left"+(i+1)))));
				keys[1][i].setText(KeyEvent.getKeyText(Integer.parseInt(props.getProperty("right"+(i+1)))));
			}

			//other properties
			fullscreen.setSelected((new Boolean(props.getProperty("fullscreen"))).booleanValue());
			players.setValue(new Integer(Integer.parseInt(props.getProperty("players"))));
			points.setValue(new Integer(Integer.parseInt(props.getProperty("maxscore"))));
			speed[Integer.parseInt(props.getProperty("speed"))].setSelected(true);
			turn[Integer.parseInt(props.getProperty("turn"))].setSelected(true);
			hole[Integer.parseInt(props.getProperty("hole"))].setSelected(true);
		}

		catch(Exception e)
		{
			//when an error occurs --> load the default properties
			loadDefaults();
		}
	}

	//save the properties
	private void saveProperties()
	{
		props.setProperty("fullscreen",""+fullscreen.isSelected());
		props.setProperty("players",players.getValue().toString());
		props.setProperty("maxscore",points.getValue().toString());
		int selectedSpeed=1;
		int selectedTurn=1;
		int selectedHole=1;
		for(int i=0;i<3;i++)
		{
			if(speed[i].isSelected()) selectedSpeed=i;
			if(turn[i].isSelected()) selectedTurn=i;
			if(hole[i].isSelected()) selectedHole=i;
		}
		props.setProperty("speed",""+selectedSpeed);
		props.setProperty("turn",""+selectedTurn);
		props.setProperty("hole",""+selectedHole);
		try
		{
			//store the properties into a file 'properties'
			props.store(new FileOutputStream("properties"),"~ CurveD ~ properties file");

		}
		catch(Exception e)
		{
			System.err.println("\nAn error occured while saving the properties.");
		}
	}

	//load the default properties
	private void loadDefaults()
	{
		try
		{
			props=new Properties();

			//load the default properties from file 'properties.defaults'
			props.load(new FileInputStream("properties.defaults"));

			//load colors and key configuration
			for(int i=0; i<4; i++)
			{
				colors[i].setBackground(new Color(Integer.parseInt(props.getProperty("color"+(i+1)))));
				keys[0][i].setText(KeyEvent.getKeyText(Integer.parseInt(props.getProperty("left"+(i+1)))));
				keys[1][i].setText(KeyEvent.getKeyText(Integer.parseInt(props.getProperty("right"+(i+1)))));
			}

			//other properties
			fullscreen.setSelected((new Boolean(props.getProperty("fullscreen"))).booleanValue());
			players.setValue(new Integer(Integer.parseInt(props.getProperty("players"))));
			points.setValue(new Integer(Integer.parseInt(props.getProperty("maxscore"))));
			speed[Integer.parseInt(props.getProperty("speed"))].setSelected(true);
			turn[Integer.parseInt(props.getProperty("turn"))].setSelected(true);
			hole[Integer.parseInt(props.getProperty("hole"))].setSelected(true);
		}
		catch(Exception e)
		{
			System.err.println("\nAn error occured while loading 'properties.defaults'");
		}
	}

	//main method
	public static void main(String[] args)
	{
		//create new Curved object
		new Curved();
	}
}