/*

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

import java.awt.*;
import java.util.*;

public class Snake
{
	//private attributes

	//current x-position
	private double x;
	//current y-position
	private double y;
	//current direction (in degrees)
	private double direction;
	//color of the snake
	private Color color;
	//counter for next allowed key event
	private int counter;
	//counter for snake holes
	private int dontDrawCounter;
	//the width of the snake
	private int width;
	//list with the latest occupied positions of the snake
	private LinkedList latest;
	//determines whether the snake is dead or not
	private boolean dead;
	//maximum listsize of the 'latest' list
	private int listSize;
	//turnspeed (0=low,1=normal,2=high)
	private int turnSpeed;
	//amount of holes (0=low,1=normal,2=high)
	private int holes;
	//timestampt of the last snakehole
	private long lastHole;

	//constructor
	public Snake(Color color, int width, int turnSpeed, int holes)
	{
		latest=new LinkedList();
		//generate random x and y position
		x=70+ Math.random()*580;
		y=70+ Math.random()*460;
		//generate a random direction (in steps of 10 degrees)
		direction=((int) (Math.random()*36))*10;
		//set other attributes
		this.color=color;
		this.width=width;
		listSize=width*width*width;
		this.turnSpeed= turnSpeed;
		this.holes=holes;
	}

	//move to the left
	public void moveLeft()
	{
		//the snake can only move when the counter is 0 or less
		if(counter<=0)
		{
			//increment the direction with 10 degrees
			direction=(direction+10)%360;
			//wait for # loops for next movement
			counter=3-turnSpeed;
		}
	}

	//move to the right
	public void moveRight()
	{
		//the snake can only move when the counter is 0 or less
		if(counter<=0)
		{
			//decrement the direction with 10 degrees
			direction=(direction-10+360)%360;
			//wait for # loops for next movement
			counter=3-turnSpeed;
		}
	}

	//update x- and y- position
	public void move()
	{
		//vertical offset = sinus
		y-=Math.sin(Math.toRadians(direction))*3;
		//horizontal offset = cosinus
		x+=Math.cos(Math.toRadians(direction))*3;

		//decrease counter
		counter--;
	}

	//getter for the x-position (rounded integer)
	public int getX()
	{
		return (int) Math.round(x);
	}

	//getter for the y-position (rounded integer)
	public int getY()
	{
		return (int) Math.round(y);
	}

	//getter for the color
	public Color getColor()
	{
		return color;
	}

	//isDead()
	public boolean isDead()
	{
		return dead;
	}

	//kill the snake
	public void kill()
	{
		dead=true;
	}

	//does the snake need to be drawn or not --> hole or not
	public boolean dontDraw()
	{
		//the snake needs to be drawn (no hole)
		if(dontDrawCounter<=0)
		{
			//generate random number
			//based on variabele holes (0=low,1=normal,2=high)
			//-> the higher 'holes' is, the higher the chance is that
			//   the snake will need to have a hole now
			int rnd = (int) (Math.random()*(40+150*(2-holes)));
			//don't draw a hole when the last hole was less than 1 second ago
			if(rnd==37 && lastHole < (System.currentTimeMillis()-1000))
			{
				dontDrawCounter = getWidth();
				//set lastHole timestamp
				lastHole = System.currentTimeMillis();
			}
			return false;
		}
		//the snake does not need to be drawn (hole)
		else
		{
			//decrease the counter
			dontDrawCounter--;
			return true;
		}
	}

	//is Point p one of the latest occupied positions of this snake?
	public boolean contains(Point p)
	{
		if(latest.contains(p))
		{
			//yes it is
			return true;
		}
		//no it isn't
		//add the point to the list
		latest.add(p);
		//trim the list to listSize
		if (latest.size()>listSize)
		{
			latest.subList(0,latest.size()-listSize).clear();
		}
		return false;
	}

	//getter for the width
	public int getWidth()
	{
		return width;
	}
}
